import { Equals, equals } from '@compositive/commons-predicates';
import { Sequence, every } from '@compositive/commons-sequences';
import { Shade } from './Shade.js';

class ColorScale extends Sequence {
    constructor() {
        super(() => this.shades[Symbol.iterator]());
    }
    get(key, outOfBoundsMethod = "none") {
        switch (outOfBoundsMethod) {
            case "wrap": {
                if (key >= 0 && key < this.length)
                    return this.shades[key];
                const lastIndex = this.length - 1;
                const absKey = Math.abs(key);
                const rounds = Math.floor(absKey / this.length);
                const offset = absKey % lastIndex;
                return this.shades[(rounds % 2 === 0 ? lastIndex : 0) - offset * (-1) ** rounds];
            }
            case "clamp": {
                return this.shades[Math.min(this.length - 1, Math.max(0, key))];
            }
            default: {
                return this.shades[key];
            }
        }
    }
    has(key) {
        return key >= 0 && key < this.length;
    }
    get length() {
        return this.shades.length;
    }
    get backmost() {
        return this.shades[0];
    }
    get midpoint() {
        const midIndex = Math.min(Math.round(this.shades.length / 2), this.shades.length - 1);
        return this.shades[midIndex];
    }
    get frontmost() {
        return this.shades[this.shades.length - 1];
    }
    filter(cb) {
        return new FixedColorScale(super.filter(cb));
    }
    [Equals](other) {
        const ColorScaleClass = this.constructor;
        return (other instanceof ColorScaleClass &&
            other.constructor === ColorScaleClass &&
            other.length === this.length &&
            equals([...other].map((t) => t.hex), [...this].map((t) => t.hex)));
    }
    buildNextWithSameElementKind(source) {
        return new FixedColorScale(source);
    }
}
class FixedColorScale extends ColorScale {
    constructor(colors) {
        super();
        this.shades = every(colors)
            .toArray()
            .map((c, i) => new Shade(this, i, c));
    }
    static async fromJson(json) {
        return new FixedColorScale(json);
    }
    reversed() {
        return new FixedColorScale([...this.shades].reverse());
    }
    async toJson() {
        const hexColors = await this.mapAsync(async (t) => t.toJson());
        return hexColors.toArray();
    }
}

export { ColorScale, FixedColorScale };
//# sourceMappingURL=ColorScale.js.map
