import { xyz, workspace, illuminant, rgb } from 'ciebase';
import { cfs, ucs, cam } from 'ciecam02';
import { ContrastLevel } from './types.js';

/* eslint-disable
  @typescript-eslint/no-unsafe-assignment,
  @typescript-eslint/no-unsafe-call,
  @typescript-eslint/no-unsafe-member-access,
  @typescript-eslint/no-unsafe-argument
*/
/** JMh correlates system (J = lightness, M = colourfulness, h = hue angle) */
const JMh = cfs("JMh");
/** CIE XYZ colorspace, using the D65 illuminant (broad daylight) */
const CieXyz = xyz(workspace.sRGB, illuminant.D60);
/** CIECAM02-UCS colorspace */
const CieCamUcs = ucs("UCS");
const whiteXYZ = CieXyz.fromRgb(rgb.fromHex("#ffffff"));
const { abs } = Math;
/**
 * Estimates contrast between two colors using calculations in the CIECAM02 space.
 * The contrast level is a number between 0 and 100.
 *
 * @param background - Background color
 * @param foreground - Foreground color
 *
 * @deprecated This will be replaced by the new SAPC contrast checker.
 */
const getCIECAM02Contrast = (background, foreground) => {
    // Convert the colors in the CIE XYZ colorspace (tristimulus values)
    const backgroundXYZ = CieXyz.fromRgb(rgb.fromHex(background.hex));
    const foregroundXYZ = CieXyz.fromRgb(rgb.fromHex(foreground.hex));
    // Initialize the CIECAM02 colorspace setting the luminance of the background to the Y
    // component of the background color and using the JMh correlates system.
    const CieCam = cam({
        // TODO: The ciecam02 library yields Infinity's and NaN's when using as background a color that
        // has 0 luminance (e.g. pure black). Using a very small luminance value instead of 0 fixes it, but
        // it should be investigated whether this can be fixed at library level.
        backgroundLuminance: Math.max(backgroundXYZ[1], 1e-100),
        adaptiveLuminance: whiteXYZ[1],
    }, JMh);
    // Convert colors from CIE XYZ to CIECAM02
    const backgroundCam = CieCam.fillOut(JMh, CieCam.fromXyz(backgroundXYZ));
    const foregroundCam = CieCam.fillOut(JMh, CieCam.fromXyz(foregroundXYZ));
    // Convert to CIECAM02-UCS (colorspace with uniform perception)
    const backgroundUcs = CieCamUcs.fromCam(backgroundCam);
    const foregroundUcs = CieCamUcs.fromCam(foregroundCam);
    const bgJp = backgroundUcs.J_p;
    const fgJp = foregroundUcs.J_p;
    // Calculate the difference in lightness
    // const deltaJ = Math.pow(Math.abs((foregroundUcs.J_p - backgroundUcs.J_p) / 100), 2) * 100;
    // const deltaJ = Math.sqrt(Math.pow(Math.abs(foregroundUcs.J_p - backgroundUcs.J_p), 2) / 0.77) * Math.sqrt(0.77);
    const deltaJ = (abs(fgJp - bgJp) / 100) ** 3.2 * 20 + 1;
    // const deltaJ = (abs(fgJ_p - bgJ_p) / (fgJ_p + bgJ_p)) ** 3.2 * 20 + 1;
    // const deltaJ = Math.pow(Math.abs(foregroundCam.J - backgroundCam.J) / 100, 4.84) * 20 + 1;
    // const deltaJ =
    //   (Math.min(
    //     (Math.max(foregroundUcs.J_p + 0.5, backgroundUcs.J_p + 0.5) /
    //       Math.min(foregroundUcs.J_p + 0.5, backgroundUcs.J_p + 0.5)) *
    //       10,
    //     100
    //   ) /
    //     100) *
    //     20 +
    //   1;
    // TODO: Is color distance a better predictor of contrast? To be tested.
    // const deltaJ = CieCamUcs.distance(backgroundUcs, foregroundUcs);
    return {
        get value() {
            return deltaJ;
        },
        get level() {
            if (deltaJ >= 7)
                return ContrastLevel.HIGH;
            if (deltaJ >= 4.5)
                return ContrastLevel.GOOD;
            if (deltaJ >= 3)
                return ContrastLevel.MINIMUM;
            return ContrastLevel.INSUFFICIENT;
        },
        toString() {
            return `${Math.round(deltaJ * 10) / 10}`;
        },
    };
};
/* eslint-enable
  @typescript-eslint/no-unsafe-assignment,
  @typescript-eslint/no-unsafe-call,
  @typescript-eslint/no-unsafe-member-access,
  @typescript-eslint/no-unsafe-argument
*/

export { getCIECAM02Contrast };
//# sourceMappingURL=getCIECAM02Contrast.js.map
